DROP TABLE IF EXISTS product_versions;
CREATE TABLE product_versions (
id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
actual BOOLEAN DEFAULT false, -- is this the actual version?
version_major INT NOT NULL,
version_minor INT NOT NULL,
version_patchlevel INT NOT NULL,
version_string VARCHAR(16) DEFAULT NULL,
description TEXT DEFAULT NULL, -- information to be dislayed to the user
major BOOLEAN DEFAULT false, -- whether this version was major
beta BOOLEAN DEFAULT false, -- is this a beta version
notification BOOLEAN DEFAULT false, -- whether to show/hide information about this update
force_display BOOLEAN DEFAULT false, -- force this update info? (only the newest such to be displayed)
release_date DATETIME
);

DROP TABLE IF EXISTS device_types;
CREATE TABLE device_types (
id INT NOT NULL PRIMARY KEY,
name VARCHAR(64) NOT NULL
);

DROP TABLE IF EXISTS agents;
CREATE TABLE agents (
id INT NOT NULL PRIMARY KEY AUTO_INCREMENT,
name VARCHAR(64) NOT NULL,
description VARCHAR(256),
minimal_version VARCHAR(32),
dir_name VARCHAR(32),
package_name VARCHAR(64),
dev_type INT,
show_in_shortcut BOOLEAN DEFAULT true,
FOREIGN KEY (dev_type) REFERENCES device_types(id)
);

DROP TABLE IF EXISTS agent_versions;
CREATE TABLE agent_versions (
id INT NOT NULL PRIMARY KEY AUTO_INCREMENT,
dev_type INT DEFAULT NULL,
version_major INT NOT NULL,
version_minor INT NOT NULL,
version_patchlevel INT NOT NULL,
actual BOOLEAN DEFAULT false,
FOREIGN KEY (dev_type) REFERENCES device_types(id)
);

CREATE TRIGGER delban_5_agent_versions
BEFORE DELETE ON agent_versions
FOR EACH ROW
SIGNAL SQLSTATE '45000' SET MESSAGE_TEXT = 'Delete not allowed on simpletable';

DROP TABLE IF EXISTS tunnel_op;
CREATE TABLE tunnel_op (
id INT NOT NULL PRIMARY KEY,
name VARCHAR(32) NOT NULL
);

DROP TABLE IF EXISTS tunnel_status;
CREATE TABLE tunnel_status (
id INT NOT NULL PRIMARY KEY,
name VARCHAR(32) NOT NULL,
description VARCHAR(64) DEFAULT NULL
);

DROP TABLE IF EXISTS tunnel_op_status_map;
CREATE TABLE tunnel_op_status_map (
id INT NOT NULL PRIMARY KEY AUTO_INCREMENT,
operation_id INT NOT NULL,
status_id INT NOT NULL,
FOREIGN KEY (operation_id) REFERENCES tunnel_op(id),
FOREIGN KEY (status_id) REFERENCES tunnel_status(id)
);

DROP TABLE IF EXISTS product_flags_license_status;
CREATE TABLE product_flags_license_status (
id INT NOT NULL PRIMARY KEY,
description VARCHAR(128)
);

DROP TABLE IF EXISTS product_flags;
CREATE TABLE product_flags (
id ENUM ('user_devchg_version', 'license_status', 'buildtag', 'about_info') NOT NULL PRIMARY KEY,
version INT,
description TEXT DEFAULT NULL
);

DROP TABLE IF EXISTS device_os_types;
CREATE TABLE device_os_types (
id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
name VARCHAR(64) NOT NULL,
dev_type INT DEFAULT NULL,
FOREIGN KEY (dev_type) REFERENCES device_types(id)
);

DROP TABLE IF EXISTS device_os_versions;
CREATE TABLE device_os_versions (
id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
os_type INT,
description VARCHAR(32),
available BOOLEAN DEFAULT false,
approved BOOLEAN DEFAULT false,
keep BOOLEAN DEFAULT false,
FOREIGN KEY (os_type) REFERENCES device_os_types(id)
);

DROP TABLE IF EXISTS fw_firmware_names;
CREATE TABLE fw_firmware_names (
id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
fwfname VARCHAR(64)
);

CREATE TRIGGER delban_1_fwfname
BEFORE DELETE ON fw_firmware_names
FOR EACH ROW
SIGNAL SQLSTATE '45000' SET MESSAGE_TEXT = 'Delete not allowed on simpletable';

DROP TABLE IF EXISTS fw_autoupgrade_policy;
CREATE TABLE fw_autoupgrade_policy (
id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
name VARCHAR(64)
);

DROP TABLE IF EXISTS device_os_autoupgrade_policy;
CREATE TABLE device_os_autoupgrade_policy (
id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
name VARCHAR(64)
);

DROP TABLE IF EXISTS typetable_value_types;
CREATE TABLE typetable_value_types (
id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
name VARCHAR(16)
);

DROP TABLE IF EXISTS agent_profiles;
CREATE TABLE agent_profiles(
id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
profile_name VARCHAR(64),
int_ping SMALLINT UNSIGNED DEFAULT 10,
int_regular INT UNSIGNED DEFAULT 260,
int_debug SMALLINT UNSIGNED DEFAULT 10,
int_fallback SMALLINT UNSIGNED DEFAULT 60,
int_fallback_debug SMALLINT UNSIGNED DEFAULT 30,
smallcycles SMALLINT UNSIGNED DEFAULT 5,
connfail_tolerance SMALLINT UNSIGNED DEFAULT 5,
regprogress SMALLINT UNSIGNED DEFAULT 100,
fw_autoupgrade INT DEFAULT 5,
os_autoupgrade INT DEFAULT 5,
system BOOLEAN DEFAULT TRUE,
FOREIGN KEY (fw_autoupgrade) REFERENCES fw_autoupgrade_policy(id),
FOREIGN KEY (os_autoupgrade) REFERENCES device_os_autoupgrade_policy(id)
);

CREATE TABLE agent_device_type_map (
id INT NOT NULL AUTO_INCREMENT,
agent_id INT NOT NULL,
device_type_id INT NOT NULL,
PRIMARY KEY (id),
FOREIGN KEY (agent_id) REFERENCES agents(id),
FOREIGN KEY (device_type_id) REFERENCES device_types(id)
);

DROP TABLE IF EXISTS liveness_time;
CREATE TABLE liveness_time (
id SMALLINT UNSIGNED NOT NULL PRIMARY KEY,
description VARCHAR(32)
);

DROP TABLE IF EXISTS liveness_misscount;
CREATE TABLE liveness_misscount (
id SMALLINT UNSIGNED NOT NULL PRIMARY KEY,
description VARCHAR(32)
);

DROP TABLE IF EXISTS fw_freshness;
CREATE TABLE fw_freshness (
id SMALLINT UNSIGNED NOT NULL PRIMARY KEY,
description VARCHAR(32)
);

DROP TABLE IF EXISTS device_timeskew_time;
CREATE TABLE device_timeskew_time (
id SMALLINT UNSIGNED NOT NULL PRIMARY KEY,
description VARCHAR(32)
);

DROP TABLE IF EXISTS user_modules;
CREATE TABLE user_modules (
id SMALLINT UNSIGNED NOT NULL PRIMARY KEY AUTO_INCREMENT,
name VARCHAR(128) NOT NULL,
versionstring VARCHAR(64) DEFAULT NULL,
internal BOOLEAN DEFAULT false,
available BOOLEAN DEFAULT false,
approved BOOLEAN DEFAULT false,
hidden BOOLEAN DEFAULT false
);

CREATE TRIGGER delban_7_ums
BEFORE DELETE ON user_modules
FOR EACH ROW
SIGNAL SQLSTATE '45000' SET MESSAGE_TEXT = 'Delete not allowed on simpletable';

DROP TABLE IF EXISTS config_profiles;
CREATE TABLE config_profiles (
id INT NOT NULL PRIMARY KEY AUTO_INCREMENT, -- profile ID
name VARCHAR(64), -- profile name; NULL for profiles from devices (default or automatic)
automatic SMALLINT UNSIGNED DEFAULT NULL, -- if specified, it specifies device to which this profile belongs
defaults BOOLEAN DEFAULT false, -- a defaults profile for OS version * fwfname; not editable
directive BOOLEAN DEFAULT true, -- true: profile is directing values to device; false: profile is updated based on profile
master SMALLINT UNSIGNED DEFAULT NULL, -- if specified, means there is a master device from which the profile is updated (overrides directive)
os_version INT, -- if default, this specifies which OS ver is this for; otherwise it specifies base OS V (full profile)
fwfname INT, -- if default profile, sets for which FWFNAME it is; otherwise like ^^^
FOREIGN KEY (os_version) REFERENCES device_os_versions(id),
FOREIGN KEY (fwfname) REFERENCES fw_firmware_names(id)
);

CREATE TRIGGER delban_4_config_profiles
BEFORE DELETE ON config_profiles
FOR EACH ROW
SIGNAL SQLSTATE '45000' SET MESSAGE_TEXT = 'Delete not allowed on simpletable';

DROP TABLE IF EXISTS config_profile_profile_map;
CREATE TABLE config_profile_profile_map (
id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
embracing INT,
included INT,
FOREIGN KEY (embracing) REFERENCES config_profiles(id) ON DELETE CASCADE,
FOREIGN KEY (included) REFERENCES config_profiles(id) ON DELETE CASCADE
);

DROP TABLE IF EXISTS config_key_groups;
CREATE TABLE config_key_groups (
id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
group_name VARCHAR(64) NOT NULL
);

CREATE TRIGGER delban_3_config_groups
BEFORE DELETE ON config_key_groups
FOR EACH ROW
SIGNAL SQLSTATE '45000' SET MESSAGE_TEXT = 'Delete not allowed on simpletable';

DROP TABLE IF EXISTS config_keys;
CREATE TABLE config_keys (
id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
key_name VARCHAR(64) NOT NULL,
group_name SMALLINT UNSIGNED NOT NULL,
value_type SMALLINT UNSIGNED NOT NULL DEFAULT 3,
FOREIGN KEY (value_type) REFERENCES typetable_value_types(id),
FOREIGN KEY (group_name) REFERENCES config_key_groups(id)
);

CREATE TRIGGER delban_2_config_keys
BEFORE DELETE ON config_keys
FOR EACH ROW
SIGNAL SQLSTATE '45000' SET MESSAGE_TEXT = 'Delete not allowed on simpletable';

DROP TABLE IF EXISTS config_values;
CREATE TABLE config_values (
id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
profile_id INT NOT NULL,
key_name SMALLINT UNSIGNED NOT NULL,
key_value TEXT DEFAULT NULL,
FOREIGN KEY (key_name) REFERENCES config_keys(id),
FOREIGN KEY (profile_id) REFERENCES config_profiles(id) ON DELETE CASCADE
);

DROP TABLE IF EXISTS devices;
CREATE TABLE devices (
id_device SMALLINT UNSIGNED PRIMARY KEY,
devid VARCHAR(4) DEFAULT NULL,
device_token VARCHAR(32) NOT NULL,
cert_token VARCHAR(32) NOT NULL,
deleted BOOLEAN DEFAULT false,
name VARCHAR(32) DEFAULT NULL,
dev_type INT,
os_ver INT,
os_ver_desired INT,
macaddr VARCHAR(12) DEFAULT NULL,
serno VARCHAR(32) DEFAULT NULL,
iccid VARCHAR(32) DEFAULT NULL, -- ICCID of first SIM card
active_sim VARCHAR(32) DEFAULT NULL, -- text from device saying which SIM card is active
fwfname INT NOT NULL DEFAULT 1,
hostname VARCHAR(64) DEFAULT NULL,
hostname_desired VARCHAR(64) DEFAULT NULL,
config_profile INT DEFAULT NULL, -- this is governing profile, 0 (NULL) by default
automatic_profile INT DEFAULT NULL, -- this is automatic profile ID (every device has automatic profile if config values are available)
uuid VARCHAR(16) DEFAULT NULL,
registered DATETIME DEFAULT CURRENT_TIMESTAMP,
gui_version INT DEFAULT 0,
agent_version INT DEFAULT NULL,
agent_profile INT DEFAULT 1,
machine_id VARCHAR(180),
FOREIGN KEY (dev_type) REFERENCES device_types(id),
FOREIGN KEY (os_ver) REFERENCES device_os_versions(id),
FOREIGN KEY (os_ver_desired) REFERENCES device_os_versions(id),
FOREIGN KEY (config_profile) REFERENCES config_profiles(id),
FOREIGN KEY (automatic_profile) REFERENCES config_profiles(id),
FOREIGN KEY (fwfname) REFERENCES fw_firmware_names(id),
FOREIGN KEY (agent_version) REFERENCES agent_versions(id),
FOREIGN KEY (agent_profile) REFERENCES agent_profiles(id)
);

ALTER TABLE config_profiles ADD FOREIGN KEY (automatic) REFERENCES devices(id_device) ON DELETE SET NULL;

ALTER TABLE config_profiles ADD FOREIGN KEY (master) REFERENCES devices(id_device) ON DELETE SET NULL;

DROP TABLE IF EXISTS device_user_modules;
CREATE TABLE device_user_modules (
id INT UNSIGNED NOT NULL PRIMARY KEY AUTO_INCREMENT,
id_device SMALLINT UNSIGNED NOT NULL, -- id of the device the UM is installed for
module SMALLINT UNSIGNED NOT NULL,
FOREIGN KEY (id_device) REFERENCES devices(id_device) ON DELETE CASCADE,
FOREIGN KEY (module) REFERENCES user_modules(id)
);

DROP TABLE IF EXISTS tunnel_ssh;
CREATE TABLE tunnel_ssh (
id INT NOT NULL PRIMARY KEY AUTO_INCREMENT,
id_device SMALLINT UNSIGNED DEFAULT NULL, -- id of the device where does the tunnel lead to
user_port INT DEFAULT 0, -- port where the user is going to connect (SSH)
device_port INT DEFAULT 0,
request_time DATETIME DEFAULT NULL,
setup_time DATETIME DEFAULT NULL,
destroy_time DATETIME DEFAULT NULL, -- last-resort desctruction time
last_transfer DATETIME DEFAULT NULL,
status INT DEFAULT 0,
requested_op INT DEFAULT 0,
FOREIGN KEY (status) REFERENCES tunnel_status(id),
FOREIGN KEY (id_device) REFERENCES devices(id_device) ON DELETE CASCADE,
FOREIGN KEY (requested_op) REFERENCES tunnel_op(id)
);

DROP TABLE IF EXISTS tunnel_web; -- table for web tunnels (to HTTPS/443 port / localhost of the device)
CREATE TABLE tunnel_web (
id INT NOT NULL PRIMARY KEY,
id_device SMALLINT UNSIGNED DEFAULT NULL, -- id of the device where does the tunnel lead to
url VARCHAR(32) DEFAULT NULL, -- part of the URL, the random one, e.g. https://white.beta.dwarftech.cz/webtunnels/<this_part> - AUTOCFG
device_port INT DEFAULT 0, -- port which does the tunnel to the device listens at - AUTOCFG 127.0.0.1:<this_port>
request_time DATETIME DEFAULT NULL, -- timestamp of the request, AUTOCFG
setup_time DATETIME DEFAULT NULL, -- when the tunnel was set up AUTOCFG
expected_time DATETIME DEFAULT NULL, -- when is the device expected to process the pending operation
destroy_time DATETIME DEFAULT NULL, -- last-resort destruction time
last_transfer DATETIME DEFAULT NULL, -- statistics AUTOCFG
status INT DEFAULT 0, -- last known status of the tunnel, AUTOCFG
requested_op INT DEFAULT 0, -- operation requested by user
FOREIGN KEY (status) REFERENCES tunnel_status(id),
FOREIGN KEY (id_device) REFERENCES devices(id_device) ON DELETE CASCADE,
FOREIGN KEY (requested_op) REFERENCES tunnel_op(id)
);

DROP TABLE IF EXISTS device_sync_statuses;
CREATE TABLE device_sync_statuses (
id SMALLINT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
description VARCHAR(32)
);

DROP TABLE IF EXISTS device_stats;
CREATE TABLE device_stats (
-- when adding field here, update mapping table to device types below
id_device SMALLINT UNSIGNED PRIMARY KEY,
livetime_class SMALLINT UNSIGNED DEFAULT 1,
livemiss_class SMALLINT UNSIGNED DEFAULT 1,
timeskew INT DEFAULT 0,
voltage FLOAT DEFAULT NULL,
temperature INT DEFAULT NULL,
sync SMALLINT UNSIGNED DEFAULT 1,
uptime INT UNSIGNED DEFAULT 0, -- device uptime (in seconds)
mem_total INT UNSIGNED DEFAULT 0, -- total device memory in KiB
mem_avail INT UNSIGNED DEFAULT 0, -- available memory (free+swap) in KiB
processes SMALLINT UNSIGNED DEFAULT 0, -- number of running processes
root_avail INT UNSIGNED DEFAULT 0, -- available space on root FS - in MiBs
loadavg1 FLOAT DEFAULT 0, -- last minute average load
loadavg5 FLOAT DEFAULT 0, -- last 5 minutes average load
loadavg15 FLOAT DEFAULT 0, -- last 15 minutes average load
last_reboot TIMESTAMP DEFAULT CURRENT_TIMESTAMP, -- timestamp of last reboot
reboot_counter INT UNSIGNED DEFAULT 0, -- total # of reboots
signal_strength SMALLINT DEFAULT NULL, -- strength of signal, typically between -0 .. -110
signal_quality SMALLINT DEFAULT NULL, -- signal quality, typically between -0 .. -30
FOREIGN KEY (id_device) REFERENCES devices(id_device) ON DELETE CASCADE,
FOREIGN KEY (livetime_class) REFERENCES liveness_time(id),
FOREIGN KEY (livemiss_class) REFERENCES liveness_misscount(id),
FOREIGN KEY (sync) REFERENCES device_sync_statuses(id)
);

DROP TABLE IF EXISTS device_stats_device_type_map;
CREATE TABLE device_stats_device_type_map (
id_type INT,
livetime_class BOOLEAN DEFAULT false,
livemiss_class BOOLEAN DEFAULT false,
timeskew BOOLEAN DEFAULT false,
voltage BOOLEAN DEFAULT false,
temperature BOOLEAN DEFAULT false,
sync BOOLEAN DEFAULT false,
uptime BOOLEAN DEFAULT false,
mem_total BOOLEAN DEFAULT false,
mem_avail BOOLEAN DEFAULT false,
processes BOOLEAN DEFAULT false,
root_avail BOOLEAN DEFAULT false,
loadavg1 BOOLEAN DEFAULT false,
loadavg5 BOOLEAN DEFAULT false,
loadavg15 BOOLEAN DEFAULT false,
last_reboot BOOLEAN DEFAULT false,
reboot_counter BOOLEAN DEFAULT false,
FOREIGN KEY (id_type) REFERENCES device_types(id)
);

DROP TABLE IF EXISTS device_netdevs;
CREATE TABLE device_netdevs(
id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
id_device SMALLINT UNSIGNED,
version INT DEFAULT 0,
label VARCHAR(32),
active BOOLEAN DEFAULT true,
mac VARCHAR(20) DEFAULT NULL,
ipaddr VARCHAR(16) DEFAULT NULL,
ipmask SMALLINT UNSIGNED DEFAULT NULL,
ip6addr VARCHAR(48) DEFAULT NULL,
ip6pref SMALLINT UNSIGNED DEFAULT NULL,
collisions BIGINT UNSIGNED DEFAULT 0,
rx_packets_mib BIGINT UNSIGNED DEFAULT 0,
rx_bytes_gib BIGINT UNSIGNED DEFAULT 0,
rx_errors BIGINT UNSIGNED DEFAULT 0,
tx_packets_mib BIGINT UNSIGNED DEFAULT 0,
tx_bytes_gib BIGINT UNSIGNED DEFAULT 0,
tx_errors BIGINT UNSIGNED DEFAULT 0,
FOREIGN KEY (id_device) REFERENCES devices(id_device) ON DELETE CASCADE
);

DROP TABLE IF EXISTS device_notes;
CREATE TABLE device_notes (
id_device SMALLINT UNSIGNED NOT NULL PRIMARY KEY,
note1 VARCHAR(64) DEFAULT NULL,
note2 VARCHAR(64) DEFAULT NULL,
note3 VARCHAR(64) DEFAULT NULL,
FOREIGN KEY (id_device) REFERENCES devices(id_device) ON DELETE CASCADE
);

DROP TABLE IF EXISTS system_labels;
CREATE TABLE system_labels (
id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
label VARCHAR(32),
description TEXT DEFAULT NULL,
UNIQUE (label)
);

DROP TABLE IF EXISTS labels;
CREATE TABLE labels (
id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
label VARCHAR(32),
UNIQUE (label)
);

DROP TABLE IF EXISTS labels_map;
CREATE TABLE labels_map (
id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
id_label INT DEFAULT NULL,
id_system_label INT DEFAULT NULL,
id_device SMALLINT UNSIGNED,
CONSTRAINT device_label UNIQUE (id_label,id_device),
FOREIGN KEY (id_label) REFERENCES labels(id),
FOREIGN KEY (id_system_label) REFERENCES system_labels(id),
FOREIGN KEY (id_device) REFERENCES devices(id_device) ON DELETE CASCADE
);

DROP TABLE IF EXISTS monitoring_groups;
CREATE TABLE monitoring_groups (
id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
name VARCHAR(64),
UNIQUE (name)
);

DROP TABLE IF EXISTS monitoring_group_device_map;
CREATE TABLE monitoring_group_device_map (
id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
id_device SMALLINT UNSIGNED,
mongrp_id SMALLINT UNSIGNED,
CONSTRAINT device_monitoring_group UNIQUE (id_device, mongrp_id),
FOREIGN KEY (id_device) REFERENCES devices(id_device) ON DELETE CASCADE,
FOREIGN KEY (mongrp_id) REFERENCES monitoring_groups(id)
);

-- following is currently replaced by dashboard/webservice and not stored in DB
-- the table will be reforged for future mongrps statistics if some persistence is needed
-- DROP TABLE IF EXISTS monitoring_group_statistics;
-- CREATE TABLE monitoring_group_statistics (
-- id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
-- name VARCHAR(32),
-- val1 INT DEFAULT 0,
-- val2 INT DEFAULT 0,
-- val3 INT DEFAULT 0,
-- val4 INT DEFAULT 0,
-- val5 INT DEFAULT 0,
-- mongrp_id SMALLINT UNSIGNED,
-- FOREIGN KEY (mongrp_id) REFERENCES monitoring_groups(id)
-- );
-- 

DROP TABLE IF EXISTS typetable_time_units;
CREATE TABLE typetable_time_units (
id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
description VARCHAR(32)
);

DROP TABLE IF EXISTS device_history_timeslot;
CREATE TABLE device_history_timeslot (
id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
description VARCHAR(32),
unit SMALLINT UNSIGNED NOT NULL,
amount SMALLINT UNSIGNED NOT NULL DEFAULT 0,
FOREIGN KEY (unit) REFERENCES typetable_time_units(id)
);

DROP TABLE IF EXISTS device_history_bit_types;
CREATE TABLE device_history_bit_types (
id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
description VARCHAR(32)
);

DROP TABLE IF EXISTS device_history_types;
CREATE TABLE device_history_types (
id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
customizable BOOLEAN DEFAULT false,
visible_dataparts INT DEFAULT 0,
description VARCHAR(64)
);

DROP TABLE IF EXISTS device_history_type_map;
CREATE TABLE device_history_type_map (
id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
history_type SMALLINT UNSIGNED,
customizable BOOLEAN DEFAULT false,
visible BOOLEAN DEFAULT true,
bit_type SMALLINT UNSIGNED,
timeslot SMALLINT UNSIGNED,
bit_order SMALLINT UNSIGNED NOT NULL,
FOREIGN KEY (history_type) REFERENCES device_history_types(id),
FOREIGN KEY (bit_type) REFERENCES device_history_bit_types(id),
FOREIGN KEY (timeslot) REFERENCES device_history_timeslot(id)
);

DROP TABLE IF EXISTS graph_types;
CREATE TABLE graph_types (
id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
description VARCHAR(64)
);

DROP TABLE IF EXISTS graph_type_selections;
CREATE TABLE graph_type_selections (
id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
graph_type SMALLINT UNSIGNED NOT NULL DEFAULT 1,
base INTEGER DEFAULT 0,
description VARCHAR(128),
FOREIGN KEY (graph_type) REFERENCES graph_types(id)
);

DROP TABLE IF EXISTS device_history_value_descriptions;
CREATE TABLE device_history_value_descriptions (
id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
description VARCHAR(128),
primary_value_type SMALLINT UNSIGNED NOT NULL,
history_type SMALLINT UNSIGNED NOT NULL,
usual_min INT DEFAULT 0,
usual_max INT DEFAULT 100,
unit VARCHAR(16),
default_ui_order SMALLINT UNSIGNED UNIQUE NOT NULL,
default_visibility BOOLEAN DEFAULT true,
default_graph_type_selection SMALLINT UNSIGNED DEFAULT 1,
actual_graph_type_selection SMALLINT UNSIGNED DEFAULT 1,
FOREIGN KEY (primary_value_type) REFERENCES typetable_value_types(id),
FOREIGN KEY (history_type) REFERENCES device_history_types(id),
FOREIGN KEY (default_graph_type_selection) REFERENCES graph_type_selections(id),
FOREIGN KEY (actual_graph_type_selection) REFERENCES graph_type_selections(id)
);

DROP TABLE IF EXISTS device_history;
CREATE TABLE device_history (
history_type SMALLINT UNSIGNED, -- type of history
id_device SMALLINT UNSIGNED NOT NULL, -- id of device (if the stored value is associated with device)
mongrp_id SMALLINT UNSIGNED, -- id of group target (if the stored value is associated with group)
value_id SMALLINT UNSIGNED NOT NULL, -- id of stored value
last_used SMALLINT UNSIGNED DEFAULT 0, -- last value that is actually filled with data (subsequent data columns are not yet populated
data_1 FLOAT DEFAULT 0,
data_2 FLOAT DEFAULT 0,
data_3 FLOAT DEFAULT 0,
data_4 FLOAT DEFAULT 0,
data_5 FLOAT DEFAULT 0,
data_6 FLOAT DEFAULT 0,
data_7 FLOAT DEFAULT 0,
data_8 FLOAT DEFAULT 0,
data_9 FLOAT DEFAULT 0,
data_10 FLOAT DEFAULT 0,
data_11 FLOAT DEFAULT 0,
data_12 FLOAT DEFAULT 0,
data_13 FLOAT DEFAULT 0,
data_14 FLOAT DEFAULT 0,
data_15 FLOAT DEFAULT 0,
data_16 FLOAT DEFAULT 0,
data_17 FLOAT DEFAULT 0,
data_18 FLOAT DEFAULT 0,
data_19 FLOAT DEFAULT 0,
data_20 FLOAT DEFAULT 0,
data_21 FLOAT DEFAULT 0,
data_22 FLOAT DEFAULT 0,
data_23 FLOAT DEFAULT 0,
data_24 FLOAT DEFAULT 0,
data_25 FLOAT DEFAULT 0,
data_26 FLOAT DEFAULT 0,
data_27 FLOAT DEFAULT 0,
data_28 FLOAT DEFAULT 0,
data_29 FLOAT DEFAULT 0,
data_30 FLOAT DEFAULT 0,
data_31 FLOAT DEFAULT 0,
data_32 FLOAT DEFAULT 0,
data_33 FLOAT DEFAULT 0,
PRIMARY KEY (id_device, value_id),
FOREIGN KEY (id_device) REFERENCES devices(id_device) ON DELETE CASCADE,
FOREIGN KEY (mongrp_id) REFERENCES monitoring_groups(id),
FOREIGN KEY (value_id) REFERENCES device_history_value_descriptions(id),
FOREIGN KEY (history_type) REFERENCES device_history_types(id)
);

DROP TABLE IF EXISTS log_event_source;
CREATE TABLE log_event_source (
id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
name VARCHAR(32)
);

DROP TABLE IF EXISTS log_event_type;
CREATE TABLE log_event_type (
id INT NOT NULL PRIMARY KEY,
source INT NOT NULL,
name VARCHAR(32),
FOREIGN KEY (source) REFERENCES log_event_source(id)
);

DROP TABLE IF EXISTS log_events;
CREATE TABLE log_events (
id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
evtype INT NOT NULL,
value INT,
source INT NOT NULL DEFAULT 1,
extra_id INT,
msg VARCHAR(256),
time DATETIME DEFAULT CURRENT_TIMESTAMP,
FOREIGN KEY (evtype) REFERENCES log_event_type(id),
FOREIGN KEY (source) REFERENCES log_event_source(id)
);

DROP TABLE IF EXISTS typetable_operators;
CREATE TABLE typetable_operators (
id SMALLINT PRIMARY KEY,
name VARCHAR(32),
oper_txt VARCHAR(10),
oper_txt2 VARCHAR(10),
oper_txt3 VARCHAR(10),
operands SMALLINT DEFAULT 1
);

DROP TABLE IF EXISTS alert_target_type;
CREATE TABLE alert_target_type (
id SMALLINT PRIMARY KEY,
name VARCHAR(32)
);

DROP TABLE IF EXISTS alert_table_selection;
CREATE TABLE alert_table_selection (
id SMALLINT PRIMARY KEY,
table_selection VARCHAR(32)
);

DROP TABLE IF EXISTS alert_target_spec;
CREATE TABLE alert_target_spec (
id SMALLINT PRIMARY KEY,
name VARCHAR(32),
valtype SMALLINT UNSIGNED NOT NULL DEFAULT 1,
seltype SMALLINT DEFAULT 0,
hint VARCHAR(64),
FOREIGN KEY (valtype) REFERENCES typetable_value_types(id),
FOREIGN KEY (seltype) REFERENCES alert_table_selection(id)
);

DROP TABLE IF EXISTS alert_target_map;
CREATE TABLE alert_target_map (
id SMALLINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
target_spec SMALLINT,
target_type SMALLINT,
FOREIGN KEY (target_type) REFERENCES alert_target_type(id),
FOREIGN KEY (target_spec) REFERENCES alert_target_spec(id)
);

DROP TABLE IF EXISTS alerts;
CREATE TABLE alerts (
id SMALLINT UNSIGNED AUTO_INCREMENT PRIMARY KEY,
name VARCHAR(64),
target_type SMALLINT DEFAULT 0, -- e.g. device, group, # devices in group, ...
target_spec SMALLINT DEFAULT 0, -- which metric/statistic
target_amount SMALLINT UNSIGNED DEFAULT 1, -- how many items out of a set (e.g. how many devices in a group)
target_device_id SMALLINT UNSIGNED DEFAULT 0, -- id of device target (if target is device)
target_mongrp_id SMALLINT UNSIGNED DEFAULT 0, -- id of group target (if target is group)
evaluate_interval INT DEFAULT 60, -- how many seconds (at least) to wait between evaluations
oper_first SMALLINT DEFAULT 0,
oper_second SMALLINT DEFAULT 0,
num_first FLOAT DEFAULT 0,
num_second FLOAT DEFAULT 0,
last_trigger_time DATETIME DEFAULT NULL,
last_notification_time DATETIME,
last_review_time DATETIME,
enabled SMALLINT UNSIGNED DEFAULT 0, -- if user want this to be checked
active SMALLINT UNSIGNED DEFAULT 0, -- dwarfgd: I am tracking actual version of the alert
counter SMALLINT UNSIGNED DEFAULT 0, -- how many times was this alert version triggered (by dwarfgd)
triggered SMALLINT UNSIGNED DEFAULT 0, -- is this triggered at the moment?
version SMALLINT UNSIGNED DEFAULT 0, -- version set by GUI
deleted BOOLEAN DEFAULT false, -- this alert was deleted
FOREIGN KEY (target_device_id) REFERENCES devices(id_device) ON DELETE CASCADE,
FOREIGN KEY (target_mongrp_id) REFERENCES monitoring_groups(id),
FOREIGN KEY (target_type) REFERENCES alert_target_type(id),
FOREIGN KEY (target_spec) REFERENCES alert_target_spec(id),
FOREIGN KEY (oper_first) REFERENCES typetable_operators(id),
FOREIGN KEY (oper_second) REFERENCES typetable_operators(id)
);

DROP TABLE IF EXISTS product_conf;
CREATE TABLE product_conf (
id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
name ENUM ('ExternalURL', 'DB version', 'isFirstRun', 'EnabledEmailNotifications', 'EnableEmailNotificationToAddress', 'NotificationEmail', 'SlackToken', 'SlackChannel', 'SenderEmail', 'SmtpServer', 'SmtpPort', 'SmtpSecurity', 'SmtpLogin', 'SmtpPassword', 'ServiceCoreName', 'WebTunnelURL', 'SysctlPath', 'ProductRootDir', 'ProductSrvDir', 'SAASMode', 'LocalPort', 'AggregatedLogsFile', 'SupportLink', 'DevicesTableColumns') NOT NULL,
valtype SMALLINT UNSIGNED DEFAULT 1,
system BOOLEAN DEFAULT false,
intval INT,
floatval FLOAT,
strval VARCHAR(512),
version INT DEFAULT 0,
time DATETIME DEFAULT CURRENT_TIMESTAMP,
FOREIGN KEY (valtype) REFERENCES typetable_value_types(id)
);

DROP TABLE IF EXISTS data_change_source;
CREATE TABLE data_change_source (
id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
name VARCHAR(64) NOT NULL
);

DROP TABLE IF EXISTS data_atom_nature;
CREATE TABLE data_atom_nature (
id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
name VARCHAR(64) NOT NULL
);

DROP TABLE IF EXISTS data_atom_context;
CREATE TABLE data_atom_context (
id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
name VARCHAR(64) NOT NULL
);

DROP TABLE IF EXISTS data_atom;
CREATE TABLE data_atom (
id SMALLINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
name VARCHAR(64) NOT NULL,
internal_type SMALLINT UNSIGNED NOT NULL DEFAULT 3,
nature SMALLINT UNSIGNED NOT NULL,
context SMALLINT UNSIGNED NOT NULL,
visible BOOLEAN DEFAULT true,
FOREIGN KEY (internal_type) REFERENCES typetable_value_types(id),
FOREIGN KEY (nature) REFERENCES data_atom_nature(id),
FOREIGN KEY (context) REFERENCES data_atom_context(id)
);

DROP TABLE IF EXISTS data_change_log;
CREATE TABLE data_change_log (
id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
source SMALLINT UNSIGNED NOT NULL, -- where did the change originated from
atom SMALLINT UNSIGNED NOT NULL, -- which data atom was changed
id_device SMALLINT UNSIGNED DEFAULT NULL, -- id of the device if value belongs to device
extra_id SMALLINT UNSIGNED DEFAULT NULL, -- another identificator if value does not belong to device
extra_txt TINYTEXT DEFAULT NULL, -- rendered textual representation of extra_id (e.g. user name)
value_txt TINYTEXT DEFAULT NULL, -- textual representation of the change
change_timestamp DATETIME DEFAULT CURRENT_TIMESTAMP,
FOREIGN KEY (source) REFERENCES data_change_source(id),
FOREIGN KEY (atom) REFERENCES data_atom(id),
FOREIGN KEY (id_device) REFERENCES devices(id_device) ON DELETE CASCADE
);

CREATE TRIGGER delban_6_dcl
BEFORE DELETE ON data_change_log
FOR EACH ROW
SIGNAL SQLSTATE '45000' SET MESSAGE_TEXT = 'Delete not allowed on simpletable';
