<?php

declare(strict_types=1);

namespace App\Controller\Admin\Ajax;

use App\Entity\Label;
use App\Form\LabelType;
use Doctrine\DBAL\Exception\UniqueConstraintViolationException;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController as Controller;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

/**
 * @Route("/admin/ajax/label", name="ajax_label_")
 */
class LabelController extends Controller
{

    /**
     * Get All Labels
     * @Route("/all", methods={"GET"}, name="get_all")
     * @param Request $request
     * @param ManagerRegistry $managerRegistry
     * @return Response
     */
    public function getAllLabelsAction(Request $request, ManagerRegistry $managerRegistry): Response {

        if ($request->isXmlHttpRequest()) {

            $labels = $managerRegistry->getRepository(Label::class)->findAll();
            $result = array();

            foreach ($labels as $label) {
                $result[$label->getId()] = ['value' => $label->getLabel()];
            }

            return new JsonResponse($result);

        }

        return new Response('Bad request.', 400);

    }

    /**
     * Create new label
     * @Route("/create", methods={"POST"}, name="create")
     * @param Request $request
     * @param ManagerRegistry $managerRegistry
     * @return Response
     */
    public function createLabelAction(Request $request, ManagerRegistry $managerRegistry): Response {

        /** @var Label $label */
        $label = new Label();
        $form = $this->createForm(LabelType::class);
        $form->handleRequest($request);
        $errors = array();

        if ($form->isSubmitted()) {
            $errorsValidator = [];
            foreach ($errorsValidator as $error) {
                array_push($errors, $error->getMessage());
            }

            if (count($errors) == 0) {

                $formData = $form->getData();
                $em = $managerRegistry->getManager();
                try {

                    $label->setLabel($formData['label']);
                    $em->persist($label);
                    $em->flush();

                }catch (UniqueConstraintViolationException $e){

                    return new JsonResponse(array(
                        'code' => 400,
                        'message' => 'error',
                        'errors' => array('error' => "Label already exists.")),
                        400);

                }catch (\Exception $e){

                    return new JsonResponse(array(
                        'code' => 400,
                        'message' => 'error',
                        'errors' => array('error' => "Unable to create the label.")),
                        400);

                }

                return new JsonResponse(array(
                    'code' => 200,
                    'message' => 'success',
                    //'errors' => array('errors' => array(''))
                ),200);
            }

        }

        return new JsonResponse(array(
            'code' => 400,
            'message' => 'error',
            'errors' => array('errors' => $errors)),
            400);
    }

    /**
     * Delete Label
     * @Route("/delete/{labelId}", methods={"GET"}, name="delete")
     * @param Request $request
     * @param int $labelId
     * @param ManagerRegistry $managerRegistry
     * @return Response
     */
    public function labelDeleteAction(Request $request, int $labelId, ManagerRegistry $managerRegistry): Response {

        if ($request->isXmlHttpRequest()) {

            $label = $managerRegistry->getRepository(Label::class)->findOneBy(['id' => $labelId]);

            if($label){

                $em = $managerRegistry->getManager();
                $em->remove($label);
                $em->flush();

            }

            return new JsonResponse(array(
                'code' => 200,
                'message' => 'Label was deleted',
                'errors' => array('errors' => array(''))),
                200);

        }

        return new Response('Bad request.', 400);

    }

}