<?php

declare(strict_types=1);

namespace App\Controller\Admin\Ajax;

use App\Entity\Device;
use App\Entity\TunnelOperation;
use App\Entity\TunnelWeb;
use App\Repository\TunnelWebRepository;
use Doctrine\Persistence\ManagerRegistry;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController as Controller;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

/**
 * @Route("/admin/ajax/tunnel", name="ajax_tunnel_")
 */
class TunnelController extends Controller
{

    /**
     * Assign Device to Web Tunnel
     * @Route("/web/assign-device", methods={"POST"}, name="web_assign_device")
     * @param Request $request
     * @param ManagerRegistry $managerRegistry
     * @return Response
     */
    public function assignDeviceToTunnel(Request $request, ManagerRegistry $managerRegistry): Response {

        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        if(isset($request->request)) {

            // Request has request data ?
            if (!isset($request->request))
                return new JsonResponse(array('status' => 'Error'),400);

            $item = (int) $request->get('item');
            $tunnelId = (int) $request->get('tunnelId');

            if (!isset($item) && !isset($tunnelId))
                return new JsonResponse(array('status' => 'Error'),400);

            $em = $managerRegistry->getManager();

            $device = $managerRegistry->getRepository(Device::class)->findOneBy(['id' => $item]);
            $tunnel = $managerRegistry->getRepository(TunnelWeb::class)->findOneBy(['id' => $tunnelId]);
            $requestedOperation = $managerRegistry
                ->getRepository(TunnelOperation::class)->findOneBy(['id' => 1]); //Setup

            if($device && $tunnel){

                $tunnel->setDevice($device);
                $tunnel->setRequestedOperation($requestedOperation);
                $em->persist($tunnel);
                $em->flush();

                return new JsonResponse(array(
                    'code' => 200,
                    'message' => 'Devices sucessfully inserted to the group.',
                    'errors' => array('errors' => array(''))),
                    200);

            }

        }

        // If we reach this point, it means that something went wrong
        return new JsonResponse(array(
            'status' => 'Error',
            'message' => 'Error'),
            400);

    }

    /**
     * Web Tunnel restart
     * @Route("/web/{webTunnelId}/restart", methods={"GET"}, name="web_restart")
     * @param Request $request
     * @param int $webTunnelId
     * @param TunnelWebRepository $tunnelWebRepository
     * @return JsonResponse|Response
     */
    public function webTunnelRestart(Request $request, int $webTunnelId, TunnelWebRepository $tunnelWebRepository): Response {

        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        if(isset($request->request)) {

            if (!isset($webTunnelId))
                return new JsonResponse(array('status' => 'Error'),400);

            $result = $tunnelWebRepository->processRequestOperation($webTunnelId, TunnelOperation::OPERATION_RESTART);

            if($result){

                return new JsonResponse(array(
                    'code' => 200,
                    'message' => 'Web tunnel was restarted.',
                    'errors' => array('errors' => array(''))),
                    200);

            }

        }

        // If we reach this point, it means that something went wrong
        return new JsonResponse(array(
            'status' => 'Error',
            'message' => 'Error'),
            400);

    }

    /**
     * Web Tunnel shutdown
     * @Route("/web/{webTunnelId}/shutdown", methods={"GET"}, name="web_shutdown")
     * @param Request $request
     * @param int $webTunnelId
     * @param TunnelWebRepository $tunnelWebRepository
     * @return JsonResponse|Response
     */
    public function webTunnelShutdown(Request $request, int $webTunnelId, TunnelWebRepository $tunnelWebRepository): Response {

        if (!$request->isXmlHttpRequest()) {
            return new Response('Bad Request', 400);
        }

        if(isset($request->request)) {

            if (!isset($webTunnelId))
                return new JsonResponse(array('status' => 'Error'),400);

            $result = $tunnelWebRepository->processRequestOperation($webTunnelId, TunnelOperation::OPERATION_SHUTDOWN);

            if($result){

                return new JsonResponse(array(
                    'code' => 200,
                    'message' => 'Web tunnel was shut down.',
                    'errors' => array('errors' => array(''))),
                    200);

            }

        }

        // If we reach this point, it means that something went wrong
        return new JsonResponse(array(
            'status' => 'Error',
            'message' => 'Error'),
            400);

    }

}