<?php

declare(strict_types=1);

namespace App\Controller\Admin\DataChangeLog;

use App\Bundles\Sg\DatatablesBundle\Response\DatatableResponse;
use App\Controller\Admin\BaseController;
use App\Datatables\DataChangeLogDatatable;
use App\Entity\DataAtom;
use Doctrine\Persistence\ManagerRegistry;
use Sg\DatatablesBundle\Datatable\DatatableFactory;
use Sg\DatatablesBundle\Datatable\DatatableInterface;
use App\Bundles\Sg\DatatablesBundle\Response\DatatableQueryBuilder;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Response;

class DataChangeLogController extends BaseController
{
    /**
     * @Route("/admin/data-change-log", name="admin_data_change_log")
     * @param Request $request
     * @param ManagerRegistry $managerRegistry
     * @param DatatableFactory $factory
     * @param DatatableResponse $responseService
     * @return Response
     * @throws \Exception
     */
    public function default(Request $request, ManagerRegistry $managerRegistry,
                            DatatableFactory $factory, DatatableResponse $responseService): Response{

        $isAjax = $request->isXmlHttpRequest();
        $this->filter = $request->query->get('column');

        $atomTypes = $managerRegistry->getRepository(DataAtom::class)->findBy(['visible' => true]);

        $datatable = $factory->create(DataChangeLogDatatable::class);
        $datatable->buildDatatable(['atomTypes' => $atomTypes]);

        if ($isAjax) {
            $responseService->setDatatable($datatable);

            $requestParams = $this->getDatatableFilter($request, $datatable);
            $datatableQueryBuilder = new DatatableQueryBuilder($requestParams, $datatable);
            $responseService->setDatatableQueryBuilder($datatableQueryBuilder);

            return $responseService->getResponse();
        }

        return $this->render('dataChangeLog/dataChangeLog.html.twig', array(
            'datatable' => $datatable,
            'filter' => $this->filter,
        ));

    }

}